/*----------------------------------------------------------------------------

	Encoder card device driver header file

--------------------------------------------------------------------------

	Revision History.

	V2.4	07/06/1997  ENCODER define replaced with VECTOR for consistency across products.
	V2.5	22/07/1997  If Win32, remove 'far' modifier.
	V2.6	11/09/1997  Pascal convention adopted.
	V2.61	27/02/1998	Add PROBE_LED_MODE PROBE_SOUND
	V2.62	15/04/1998	added footswitch etc
	V2.7	25/04/1998	Added FN_DEF and PTR_DEF,Added define for SSI MODE settings
	V2.8	04/06/1998	Added EXT_DEF for C++
	V2.9	25/06/2001	Added encoder mode function and changed define to enum
	V3.0	18/04/2002	Added functions to support additional PCI features
	V3.1	10/10/2002	Added support for MS Windows callbacks
						Added functions enclib_callback and NUM_LOSTCALLBACKS
						Renamed function PROBE_VECT to PROBE_INT_VECT
	V3.2	22/10/2004	Added PULSEGEN0_TIME, PULSEGEN0_SEQ_EN and PULSEGEN0_TIMEOUT
----------------------------------------------------------------------------*/

#if !defined(__DEVLIB3)
#define __DEVLIB3
#endif

#if !defined(__ENCODER3)
#define __ENCODER3

enum COMMANDS
{
	VECTOR=0,NUM_AXES,NUM_TIMERS,NUM_INPUTS,NUM_DACS,NUM_OUTPUTS,NUM_COLLISIONS,NUM_BOARDS,CARD_TYPE,VERSION_NUM,
	CNT_16=10,MODE,AXIS_SIZE,ENCODER_TYPE,AXIS_INPUTS,AXIS_STATUS,AXIS_OUT_EN,
	MARK_16=20,MARK_INPUT,MARK_INT,MARK_FUNC,MARK_INT_VECT,MARK_INT_OCCUR,MARK_LATCH_SEL,MARK_OUT_EN,
	ZERO_INPUT=30,ZERO_INT,ZERO_FUNC,ZERO_INT_VECT,ZERO_INT_OCCUR,
	AXIS_32=40,MARK_32,VEL_INST,VEL_FILT,ACCEL_INST,ACCEL_FILT,PROBE_32,ABSOLUTE_32,TIMER_32,
	INPUT=50,
	TIMER=60,TIMER_INT,TIMER_INT_VECT,TIMER_INT_OCCUR,TIMER_LATCH_SEL,TIMER1_FUNC,TIMER_OUT_EN,
	PROBE_16=90,PROBE_INPUT,PROBE_INT,PROBE_FUNC,PROBE_INT_VECT,PROBE_INT_OCCUR,PROBE_SENSE,PROBE_LED,PROBE_SOUND,PROBE_FOOTSWITCH,
	PROBE_LATCH_SEL=100,PROBE_OUT_EN,
	TIMESTAMP_NOW=110,TIMESTAMP_EVENT,TIMESTAMP_SEL,
	PULSEGEN0=120,PULSEGEN0_MODE,PULSEGEN0_EN,PULSEGEN0_OCCUR,PULSEGEN0_OUT_EN,PULSEGEN0_INT_VECT,PULSEGEN0_TIME,PULSEGEN0_SEQ_EN,PULSEGEN0_TIMEOUT,
	COMPARE0=130,COMPARE0_FUNC,COMPARE0_OCCUR,COMPARE0_OUT_EN,COMPARE0_INT_VECT,COMPARE1,COMPARE1_FUNC,COMPARE1_OCCUR,COMPARE1_INT_VECT=139,
	USEREVENT0_OCCUR=140,USEREVENT0_OUT_EN,
	BOARD_INPUT_EN=150,BOARD_INPUT_OCCUR,BOARD_INPUT_INT_VECT,AXIS_INPUT_EN=155,AXIS_INPUT_OCCUR,AXIS_INPUT_INT_VECT,
	IO=160,IO_DIR,IO_32=165,IO_32_DIR,NUM_IOS=169,
	BUF_CLK_CHANNEL=170,BUF_CLK_TYPE,BUF_CLK_DIV,BUF_NUM_DATA,BUF_DATA_INDEX,BUF_DATA_CHANNEL,BUF_DATA_TYPE,
	BUF_SIZE=180,BUF_MODE,BUF_EN,BUF_FLUSH,BUF_STATUS,BUF_READ,BUF_MEMFREE=187,BUF_MAXDATA,NUM_BUFFERS,
	NUM_LOSTCALLBACKS=200
};

enum DEFINTS {SPARE_INT,COUNTER_INT,ZERO_Z_INT,MARK_Z_INT,ZERO_Y_INT,MARK_Y_INT,ZERO_X_INT,MARK_X_INT};
enum LEDS {PROBE_LED_OFF,PROBE_LED_ON,PROBE_LED_AUTO};
enum SOUND {PROBE_SOUND_OFF,PROBE_SOUND_ON};
enum FOOTSWITCH {FOOTSWITCH_OFF,FOOTSWITCH_AUTO};

#define FILTER_SAMPLES 10

// Axis Mode Definitions
union MODE_TYPE
{
	unsigned long val;
	struct MODE_INC_BITS_TYPE
	{
		unsigned int MODE : 4; // 00000000 00000000 00000000 0000????
		unsigned int INVA : 1; // 00000000 00000000 00000000 000?0000
		unsigned int INVB : 1; // 00000000 00000000 00000000 00?00000
		unsigned int INVM : 1; // 00000000 00000000 00000000 0?000000
		unsigned int INVZ : 1; // 00000000 00000000 00000000 ?0000000
		unsigned int MSD  : 1; // 00000000 00000000 0000000? 00000000
	} inc_bits;
	struct MODE_SSI_BITS_TYPE
	{
		unsigned int FREQ   : 3; // 00000000 00000000 00000000 00000???
		unsigned int        : 5; // 00000000 00000000 00000000 ?????000
		unsigned int OFFSET : 4; // 00000000 00000000 0000???? 00000000
		unsigned int CHECK  : 2; // 00000000 00000000 00??0000 00000000
		unsigned int CODE   : 1; // 00000000 00000000 0?000000 00000000
		unsigned int OUTPUT : 1; // 00000000 00000000 ?0000000 00000000
		unsigned int OPER   : 3; // 00000000 00000??? 00000000 00000000
		unsigned int        : 4; // 00000000 0????000 00000000 00000000
		unsigned int RCIE   : 1; // 00000000 ?0000000 00000000 00000000
		unsigned int LENGTH : 6; // 00?????? 00000000 00000000 00000000
	} ssi_bits;
};
enum SSI_MODE_FREQ_TYPE
{
	SSI_MODE_FREQ_2M50Hz,
	SSI_MODE_FREQ_1M25Hz,
	SSI_MODE_FREQ_625kHz,
	SSI_MODE_FREQ_313kHz,
	SSI_MODE_FREQ_156kHz,
	SSI_MODE_FREQ_78k1Hz
};
enum SSI_MODE_CHECK_TYPE
{
	SSI_MODE_CHECK_POWER_FAIL=1,
	SSI_MODE_CHECK_PARITY_EVEN,
	SSI_MODE_CHECK_PARITY_ODD
};
enum SSI_MODE_CODE_TYPE
{
	SSI_MODE_CODE_BINARY,
	SSI_MODE_CODE_GRAY
};
enum SSI_MODE_OPER_TYPE
{
	SSI_MODE_OPER_ONESHOT,
	SSI_MODE_OPER_TIMED,
	SSI_MODE_OPER_CONTINUOUS,
	SSI_MODE_OPER_CHANNEL_DI0=4
};

// Axis Inputs Definitions
union INPUTS_TYPE
{
	unsigned long val;
	struct INC_BITS_TYPE
	{
		unsigned int A    : 1; // 00000000 00000000 00000000 0000000?
		unsigned int B    : 1; // 00000000 00000000 00000000 000000?0
		unsigned int M    : 1; // 00000000 00000000 00000000 00000?00
		unsigned int Z    : 1; // 00000000 00000000 00000000 0000?000
		unsigned int LIM0 : 1; // 00000000 00000000 00000000 000?0000
		unsigned int LIM1 : 1; // 00000000 00000000 00000000 00?00000
	} inc_bits;
	struct SSI_BITS_TYPE
	{
		unsigned int DATA : 1; // 00000000 00000000 00000000 0000000?
	} ssi_bits;
};
// Axis Status Definitions
union STATUS_TYPE
{
	unsigned long val;
	struct STATUS_INC_BITS_TYPE
	{
		unsigned int PWRF  : 1; // 00000000 00000000 00000000 0000000?
		unsigned int QUADF : 1; // 00000000 00000000 00000000 000000?0
	}inc_bits;
	struct STATUS_SSI_BITS_TYPE
	{
		unsigned int PWRF : 1; // 00000000 00000000 00000000 0000000?
		unsigned int PARF : 1; // 00000000 00000000 00000000 000000?0
	}ssi_bits;
};
// Latch Selector Definitions
enum TIMER_LATCH_SEL_TYPE
{
	TLS_TIMER1_SYNCOCCUR,
	TLS_READ_COUNTER,
	TLS_PULSEGEN0_SYNCOCCUR,
	TLS_USEREVENT0_OCCUR
};
enum MARK_LATCH_SEL_TYPE
{
	MLS_MARKER_OCCUR,
	MLS_ZERO_OCCUR,
	MLS_PULSEGEN0_SYNCOCCUR,
	MLS_USEREVENT0_OCCUR,
	MLS_CHANNEL_DI0_OCCUR,
	MLS_AXISCOMP0_SYNCOCCUR,
	MLS_AXISCOMP1_SYNCOCCUR,
};
enum PROBE_LATCH_SEL_TYPE
{
	PLS_PROBE_OCCUR,
	PLS_READ_COUNTER,
	PLS_PULSEGEN0_SYNCOCCUR,
	PLS_USEREVENT0_OCCUR,
	PLS_CHANNEL_DI0_OCCUR,
	PLS_AXISCOMP0_SYNCOCCUR,
	PLS_AXISCOMP1_SYNCOCCUR,
};

// Time-stamper Selector Definitions
enum TIMESTAMPER_SEL_TYPE
{
	TSTS_PROBE_OCCUR,
	TSTS_TIMER1_SYNCOCCUR,
	TSTS_USEREVENT0_OCCUR,
	TSTS_BOARD_DI0_OCCUR,
	TSTS_PULSEGEN0_SYNCOCCUR,
	TSTS_AXISCOMP0_SYNCOCCUR,
	TSTS_AXISCOMP1_SYNCOCCUR,
};

// Pulse Generator Definitions
union PULSEGEN_MODE_TYPE
{
	unsigned long val;
	struct PULSEGEN_MODE_BITS_TYPE
	{
		unsigned int DIR    : 1; // 00000000 00000000 00000000 0000000?
		unsigned int BIDIR  : 1; // 00000000 00000000 00000000 000000?0
		unsigned int HWCTRL : 1; // 00000000 00000000 00000000 00000?00
		unsigned int DEGLCH : 1; // 00000000 00000000 00000000 0000?000
	} bits;
};
enum PULSEGEN_MODE_DIR_TYPE
{
	PULSEGEN_MODE_DIR_NEG,
	PULSEGEN_MODE_DIR_POS
};

// Buffer Definitions
enum BUF_CLK_TYPE
{
	BUF_CLK_TIMER_OCCUR,
	BUF_CLK_PROBE_OCCUR,
	BUF_CLK_BOARD_DI0_OCCUR,
	BUF_CLK_USEREVENT0_OCCUR,
	BUF_CLK_MARKER_OCCUR = 16,
	BUF_CLK_ZERO_OCCUR,
	BUF_CLK_PULSEGEN0_OCCUR,
	BUF_CLK_AXISCOMP0_OCCUR,
	BUF_CLK_AXISCOMP1_OCCUR,
	BUF_CLK_CHAN_DI0_OCCUR,
	BUF_CLK_SSI_READ_COMPLETE_OCCUR
};
enum BUF_DATA_TYPE
{
	BUF_DATA_BUFFER_CLOCK_COUNTER,
	BUF_DATA_TIMESTAMP_NOW,
	BUF_DATA_TIMESTAMP_EVENT,
	BUF_DATA_IO_32,
	BUF_DATA_TIMER_LATCH = 16,
	BUF_DATA_SSI_LATCH = 16,
	BUF_DATA_MARKER_LATCH,
	BUF_DATA_PROBE_LATCH,
	BUF_DATA_AXIS_32
};
union BUF_MODE_TYPE
{
	unsigned long val;
	struct BUF_MODE_BITS_TYPE
	{
		unsigned int LOG  : 1; // 00000000 00000000 00000000 0000000?
	} bits;
};
enum BUF_MODE_LOG_TYPE
{
	BUF_MODE_LOG_DISCARDNEWDATA,
	BUF_MODE_LOG_DISCARDOLDDATA
};

/*----------------------------------------------------------------------------

	Function definitions.

----------------------------------------------------------------------------*/
#ifdef _WIN32

#define	PTR_DEF
#define	FN_DEF __stdcall

#else

#define	PTR_DEF far
#define	FN_DEF _pascal

#endif

#ifdef __cplusplus

#define EXT_DEF extern "C"

#else

#define EXT_DEF

#endif	

EXT_DEF char PTR_DEF * FN_DEF enclib_version(void);

EXT_DEF long FN_DEF read_encoder(short command,short channel);
EXT_DEF long FN_DEF read_encoder_ext(void);

EXT_DEF void FN_DEF write_encoder(short command,short channel,long value);
EXT_DEF void FN_DEF write_encoder_ext(long value);

EXT_DEF short FN_DEF open_encoder(void);
EXT_DEF void FN_DEF close_encoder(void);

EXT_DEF short FN_DEF enclib_callback(short receive,long priority);
// User function should be defined as void myfunction(void)

/*----------------------------------------------------------------------------

	Obsolete definitions.

----------------------------------------------------------------------------*/

// Axis Mode Legacy Definitions
enum MODES {UPDWN,QUADx1A,QUADx1B,QUADx2A,QUADx2B,QUADx4AB,PULSE,FREQ};
enum SSI_FREQUENCY {MODE_FREQ_2M5Hz,MODE_FREQ_1M25Hz,MODE_FREQ_625kHz,MODE_FREQ_312kHz,MODE_FREQ_156kHz,MODE_FREQ_78kHz};
enum SSI_CHECK {MODE_POWER_FAIL=0x00001000,MODE_PARITY_EVEN=0x00002000,MODE_PARITY_ODD=0x00003000};
enum SSI_CODE {MODE_BINARY,MODE_GRAY=0x00004000};

#define MODE_LENGTH_ADJUST	0x01000000
#define MODE_OFFSET_ADJUST	0x00000100
#define	MODE_CHECK_ADJUST	0x00001000
#define MODE_CODE_ADJUST	0x00004000
#define MODE_OPER_ADJUST	0x00010000
#define MODE_OUT_ADJUST		0x00008000

#define MODE_LENGTH_MASK	0x3f000000
#define MODE_OFFSET_MASK	0x00000f00
#define MODE_FREQ_MASK		0x00000007
#define MODE_CODE_MASK		0x00004000
#define MODE_CHK_MASK		0x00003000
#define MODE_OPER_MASK		0x00030000
#define MODE_OUT_MASK		0x00008000

#define MODE_ONE_SHOT		0x00000000
#define MODE_TIMED			0x00010000
#define MODE_CONTINUE		0x00020000

#endif


/*----------------------------------------------------------------------------

	End.

----------------------------------------------------------------------------*/
